SMOTENC — Version 0.11.0.dev0

# SMOTENC

class imblearn.over_sampling.SMOTENC(categorical_features, *, categorical_encoder=None, sampling_strategy='auto', random_state=None, k_neighbors=5, n_jobs=None)

  Synthetic Minority Over-sampling Technique for Nominal and Continuous.

  Unlike `SMOTE`, SMOTE-NC for dataset containing numerical and categorical features. However, it is not designed to work with only categorical features.

  Read more in the User Guide.

  New in version 0.4.

  Parameters
    categorical_featuresarray-like of shape (n_cat_features,) or (n_features,)

      Specified which features are categorical. Can either be:

      - array of indices specifying the categorical features;

      - mask array of shape (n_features, ) and `bool` dtype for which `True` indicates the categorical features.

    categorical_encoderestimator, default=None

      One-hot encoder used to encode the categorical features. If `None`, a `OneHotEncoder` is used with default parameters apart from `handle_unknown` which is set to ‘ignore’.

    sampling_strategyfloat, str, dict or callable, default=’auto’

      Sampling information to resample the data set.

      - When `float`, it corresponds to the desired ratio of the number of samples in the minority class over the number of samples in the majority class after resampling. Therefore, the ratio is expressed as \(\alpha_{os} = N_{rm} / N_{M}\) where \(N_{rm}\) is the number of samples in the minority class after resampling and \(N_{M}\) is the number of samples in the majority class.

        > Warning

        > `float` is only available for binary classification. An error is raised for multi-class classification.

      - When `str`, specify the class targeted by the resampling. The number of samples in the different classes will be equalized. Possible choices are:

        > `'minority'`: resample only the minority class;

        > `'not minority'`: resample all classes but the minority class;

        > `'not majority'`: resample all classes but the majority class;

        > `'all'`: resample all classes;

        > `'auto'`: equivalent to `'not majority'`.

      - When `dict`, the keys correspond to the targeted classes. The values correspond to the desired number of samples for each targeted class.

      - When callable, function taking `y` and returns a `dict`. The keys correspond to the targeted classes. The values correspond to the desired number of samples for each class.

    random_stateint, RandomState instance, default=None

      Control the randomization of the algorithm.

      - If int, `random_state` is the seed used by the random number generator;

      - If `RandomState` instance, random_state is the random number generator;

      - If `None`, the random number generator is the `RandomState` instance used by `np.random`.

    k_neighborsint or object, default=5

      The nearest neighbors used to define the neighborhood of samples to use to generate the synthetic samples. You can pass:

      - an `int` corresponding to the number of neighbors to use. A `~sklearn.neighbors.NearestNeighbors` instance will be fitted in this case.

      - an instance of a compatible nearest neighbors algorithm that should implement both methods `kneighbors` and `kneighbors_graph`. For instance, it could correspond to a `NearestNeighbors` but could be extended to any compatible class.

    n_jobsint, default=None

      Number of CPU cores used during the cross-validation loop. `None` means 1 unless in a `joblib.parallel_backend` context. `-1` means using all processors. See Glossary for more details.

      Deprecated since version 0.10: `n_jobs` has been deprecated in 0.10 and will be removed in 0.12. It was previously used to set `n_jobs` of nearest neighbors algorithm. From now on, you can pass an estimator where `n_jobs` is already set instead.

  Attributes
    sampling_strategy_dict

      Dictionary containing the information to sample the dataset. The keys corresponds to the class labels from which to sample and the values are the number of samples to sample.

    nn_k_estimator object

      Validated k-nearest neighbours created from the `k_neighbors` parameter.

    `ohe_` `OneHotEncoder`

      One-hot encoder used to encode the categorical features.

    categorical_encoder_estimator

      The encoder used to encode the categorical features.

    categorical_features_ndarray of shape (n_cat_features,), dtype=np.int64

      Indices of the categorical features.

    continuous_features_ndarray of shape (n_cont_features,), dtype=np.int64

      Indices of the continuous features.

    median_std_float

      Median of the standard deviation of the continuous features.

    n_features_int

      Number of features observed at `fit`.

    n_features_in_int

      Number of features in the input dataset.

      New in version 0.9.

    feature_names_in_ndarray of shape (`n_features_in_`,)

      Names of features seen during `fit`. Defined only when `X` has feature names that are all strings.

      New in version 0.10.

  See also

  `SMOTE`

    Over-sample using SMOTE.

  `SMOTEN`

    Over-sample using the SMOTE variant specifically for categorical features only.

  `SVMSMOTE`

    Over-sample using SVM-SMOTE variant.

  `BorderlineSMOTE`

    Over-sample using Borderline-SMOTE variant.

  `ADASYN`

    Over-sample using ADASYN.

  `KMeansSMOTE`

    Over-sample applying a clustering before to oversample using SMOTE.

  Notes

  See the original paper [1] for more details.

  Supports multi-class resampling. A one-vs.-rest scheme is used as originally proposed in [1].

  See Compare over-sampling samplers, and Sample generator used in SMOTE-like samplers.

  References

  1(1,2)

    N. V. Chawla, K. W. Bowyer, L. O.Hall, W. P. Kegelmeyer, “SMOTE: synthetic minority over-sampling technique,” Journal of artificial intelligence research, 321-357, 2002.

  Examples

      >>> from collections import Counter
      >>> from numpy.random import RandomState
      >>> from sklearn.datasets import make_classification
      >>> from imblearn.over_sampling import SMOTENC
      >>> X, y = make_classification(n_classes=2, class_sep=2,
      ... weights=[0.1, 0.9], n_informative=3, n_redundant=1, flip_y=0,
      ... n_features=20, n_clusters_per_class=1, n_samples=1000, random_state=10)
      >>> print(f'Original dataset shape {X.shape}')
      Original dataset shape (1000, 20)
      >>> print(f'Original dataset samples per class {Counter(y)}')
      Original dataset samples per class Counter({1: 900, 0: 100})
      >>> # simulate the 2 last columns to be categorical features
      >>> X[:, -2:] = RandomState(10).randint(0, 4, size=(1000, 2))
      >>> sm = SMOTENC(random_state=42, categorical_features=[18, 19])
      >>> X_res, y_res = sm.fit_resample(X, y)
      >>> print(f'Resampled dataset samples per class {Counter(y_res)}')
      Resampled dataset samples per class Counter({0: 900, 1: 900})

  Methods

  `fit`(X, y) | Check inputs and statistics of the sampler.
  `fit_resample`(X, y) | Resample the dataset.
  `get_feature_names_out`([input_features]) | Get output feature names for transformation.
  `get_metadata_routing`() | Get metadata routing of this object.
  `get_params`([deep]) | Get parameters for this estimator.
  `set_params`(**params) | Set the parameters of this estimator.

  fit(X, y)

    Check inputs and statistics of the sampler.

    You should use `fit_resample` in all cases.

    Parameters
      X{array-like, dataframe, sparse matrix} of shape (n_samples, n_features)

        Data array.

      yarray-like of shape (n_samples,)

        Target array.

    Returns
      selfobject

        Return the instance itself.

  fit_resample(X, y)

    Resample the dataset.

    Parameters
      X{array-like, dataframe, sparse matrix} of shape (n_samples, n_features)

        Matrix containing the data which have to be sampled.

      yarray-like of shape (n_samples,)

        Corresponding label for each sample in X.

    Returns
      X_resampled{array-like, dataframe, sparse matrix} of shape (n_samples_new, n_features)

        The array containing the resampled data.

      y_resampledarray-like of shape (n_samples_new,)

        The corresponding label of `X_resampled`.

  get_feature_names_out(input_features=None)

    Get output feature names for transformation.

    Parameters
      input_featuresarray-like of str or None, default=None

        Input features.

        - If `input_features` is `None`, then `feature_names_in_` is used as feature names in. If `feature_names_in_` is not defined, then the following input feature names are generated: `["x0", "x1", ..., "x(n_features_in_ - 1)"]`.

        - If `input_features` is an array-like, then `input_features` must match `feature_names_in_` if `feature_names_in_` is defined.

    Returns
      feature_names_outndarray of str objects

        Same as input features.

  get_metadata_routing()

    Get metadata routing of this object.

    Please check User Guide on how the routing mechanism works.

    Returns
      routingMetadataRequest

        A `MetadataRequest` encapsulating routing information.

  get_params(deep=True)

    Get parameters for this estimator.

    Parameters
      deepbool, default=True

        If True, will return the parameters for this estimator and contained subobjects that are estimators.

    Returns
      paramsdict

        Parameter names mapped to their values.

  property ohe_

    One-hot encoder used to encode the categorical features.

  set_params(**params)

    Set the parameters of this estimator.

    The method works on simple estimators as well as on nested objects (such as `Pipeline`). The latter have parameters of the form `<component>__<parameter>` so that it’s possible to update each component of a nested object.

    Parameters
      **paramsdict

        Estimator parameters.

    Returns
      selfestimator instance

        Estimator instance.

## Examples using `imblearn.over_sampling.SMOTENC`

Compare over-sampling samplers

Compare over-sampling samplers

previous

SMOTE

next

SMOTEN